#include "vl6180x_glue.h"

#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>

extern "C" {
#include <i2c/smbus.h>
}

#include <ros/ros.h>

#include "vl6180x_i2c.h"

static int i2c_dev_file;

void VL6180x_glue_init(const char *i2c_dev_path) {
  i2c_dev_file = open(i2c_dev_path, O_RDWR);
  if (i2c_dev_file < 0) {
    ROS_FATAL("Couldn't open %s: %s", i2c_dev_path, strerror(errno));
    return;
  }

  if (ioctl(i2c_dev_file, I2C_SLAVE, VL6180X_I2C_ADDR) < 0) {
    close(i2c_dev_file);
    ROS_FATAL("Couldn't set slave address: %s", strerror(errno));
    return;
  }

  ROS_INFO("Successfully opened I2C device & set address");
}

extern "C" {

int VL6180x_I2CWrite(VL6180xDev_t dev, uint8_t *buff, uint8_t len) {
  int ret = write(i2c_dev_file, buff, len);
  if (ret == len) {
    return 0;
  } else if (ret < 0) {
    ROS_ERROR("Error while writing to I2C device: %s", strerror(errno));
    return -1;
  } else {
    ROS_ERROR("Only wrote %d bytes (of %d requested)", ret, len);
  }
  return -1;
}

int VL6180x_I2CRead(VL6180xDev_t dev, uint8_t *buff, uint8_t len) {
  int ret = read(i2c_dev_file, buff, len);
  if (ret == len) {
    return 0;
  } else if (ret < 0) {
    ROS_ERROR("Error while reading from I2C device: %s", strerror(errno));
  } else {
    ROS_ERROR("Only read %d bytes (of %d requested)", ret, len);
  }
  return -1;
}
}
